@@ -2,12 +2,12 @@
 
    \brief ScanCache (anti-virus scan result caching) routines
 
-   $Id: scancache.cxx,v 1.4 2007/02/12 00:21:20 burghardt Exp $
+   $Id: scancache.cxx,v 1.7 2008-11-23 20:50:00 burghardt Exp $
 
 *//*
 
    ClamFS - An user-space anti-virus protected file system
-   Copyright (C) 2007 Krzysztof Burghardt.
+   Copyright (C) 2007,2008 Krzysztof Burghardt.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,12 +24,20 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#include <scancache.hxx>
+#include "scancache.hxx"
 
 namespace clamfs {
 
+CachedResult::CachedResult(bool isFileClean, time_t scanFileTimestamp) {
+    isClean = isFileClean;
+    scanTimestamp = scanFileTimestamp;
+}
+
+CachedResult::~CachedResult() {
+}
+
 ScanCache::ScanCache(long int elements, long int expire):
-    ExpireLRUCache<ino_t, time_t>::ExpireLRUCache<ino_t, time_t>(elements, expire) {
+    ExpireLRUCache<ino_t, CachedResult>::ExpireLRUCache<ino_t, CachedResult>(elements, expire) {
 }
 
 ScanCache::~ScanCache() {
